from typing import Dict, Any, Optional

from checkov.terraform.tag_providers import aws
from checkov.terraform.tag_providers import azure
from checkov.terraform.tag_providers import gcp

provider_tag_mapping = {"aws": aws.get_resource_tags, "azure": azure.get_resource_tags, "gcp": gcp.get_resource_tags}


def get_resource_tags(resource_type: str, entity_config: Dict[str, Any]) -> Optional[Dict[str, Any]]:
    if not isinstance(entity_config, dict):
        return None

    if "_" not in resource_type:
        return None  # probably not a resource block
    provider = resource_type[: resource_type.index("_")]
    provider_tag_function = provider_tag_mapping.get(provider)
    if provider_tag_function:
        return provider_tag_function(entity_config)
    else:
        return None



from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AbsGooglePostgresqlDatabaseFlags(BaseResourceCheck):
    def __init__(self, name, id, categories, supported_resources, flag_name, flag_values):
        super().__init__(name, id, categories, supported_resources)
        self.flag_name = flag_name
        self.flag_values = flag_values

    def scan_resource_conf(self, conf):
        if 'database_version' in conf.keys() and isinstance(conf['database_version'][0], str) and 'POSTGRES' in conf['database_version'][0]:
            if 'settings' in conf.keys():
                self.evaluated_keys = ['database_version/[0]/POSTGRES', 'settings']
                flags = conf['settings'][0].get('database_flags')
                if flags:
                    evaluated_keys_prefix = 'settings/[0]/database_flags'
                    if isinstance(flags[0], list):
                        # treating use cases of the following database_flags parsing
                        # (list of list of dictionaries with strings):'database_flags':
                        # [[{'name': '<key>', 'value': '<value>'}, {'name': '<key>', 'value': '<value>'}]]
                        flags = flags[0]
                        evaluated_keys_prefix += '/[0]'
                    else:
                        # treating use cases of the following database_flags parsing
                        # (list of dictionaries with arrays): 'database_flags':
                        # [{'name': ['<key>'], 'value': ['<value>']},{'name': ['<key>'], 'value': ['<value>']}]
                        flags = [{key: flag[key][0] for key in flag} for flag in flags]
                    for flag in flags:
                        if isinstance(flag, dict) and flag['name'] == self.flag_name and flag['value'] in self.flag_values:
                            self.evaluated_keys = ['database_version/[0]/POSTGRES',
                                                   f'{evaluated_keys_prefix}/[{flags.index(flag)}]/name',
                                                   f'{evaluated_keys_prefix}/[{flags.index(flag)}]/value']
                            return CheckResult.PASSED
                    self.evaluated_keys = ['database_version/[0]/POSTGRES', 'settings/[0]/database_flags']
            return CheckResult.FAILED
        return CheckResult.UNKNOWN



from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_root_container_check import BaseK8sRootContainerCheck


class RootContainers(BaseK8sRootContainerCheck):
    def __init__(self):
        # CIS-1.3 1.7.6
        # CIS-1.5 5.2.6
        name = "Minimize the admission of root containers"
        # Check runAsNonRoot.  If false, then ensure runAsUser > 0
        # Location: Pod.spec.runAsUser / runAsNonRoot
        # Location: CronJob.spec.jobTemplate.spec.template.spec.securityContext.runAsUser / runAsNonRoot
        # Location: *.spec.template.spec.securityContext.runAsUser / runAsNonRoot
        id = "CKV_K8S_23"
        super().__init__(name=name, id=id)

    def scan_spec_conf(self, conf):
        spec = self.extract_spec(conf)

        # Collect results
        if spec and isinstance(spec, dict):
            results = {"pod": {}, "container": []}
            results["pod"]["runAsNonRoot"] = self.check_runAsNonRoot(spec)
            results["pod"]["runAsUser"] = self.check_runAsUser(spec, 1)

            containers = spec.get("containers", [])
            if not isinstance(containers, list):
                return CheckResult.UNKNOWN
            for c in containers:
                cresults = {"runAsNonRoot": self.check_runAsNonRoot(c), "runAsUser": self.check_runAsUser(c, 1)}
                results["container"].append(cresults)

            # Evaluate pass / fail
            # Container values override Pod values
            # Pod runAsNonRoot == True, plus no override at container spec   (PASSED)
            # Pod runAsNonRoot == True, but container runAsNonRoot == False
            #                     If runAsUser failed or absent (FAILED)
            #                     if runAsUser passed, the check will pass (but don't want to pass one container if another fails)
            if results["pod"]["runAsNonRoot"] == "PASSED":
                for cr in results["container"]:
                    if cr["runAsNonRoot"] == "FAILED":
                        if cr["runAsUser"] == "FAILED" or cr["runAsUser"] == "ABSENT":
                            return CheckResult.FAILED
                return CheckResult.PASSED
            elif results["pod"]["runAsUser"] == "PASSED":
                # Pod runAsNonRoot == False (or absent) ; Pod runAsUser > 0 (PASSED)
                # If container runAsUser FAILED, then overall fail as it overrides pod (FAILED)
                for cr in results["container"]:
                    if cr["runAsUser"] == "FAILED":
                        return CheckResult.FAILED
                return CheckResult.PASSED
            else:
                # Pod runAsNonRoot and runAsUser failed or absent
                #   If container runAsNonRoot true (PASSED)
                #   If container runAsNonRoot failed or absent, but runAsUser passed (PASSED)
                #   If container runAsNonRoot failed or absent, but runAsUser failed/absent (FAILED)
                for cr in results["container"]:

                    if cr["runAsNonRoot"] == "PASSED":
                        continue
                    if cr["runAsNonRoot"] == "FAILED" or cr["runAsNonRoot"] == "ABSENT":
                        if cr["runAsUser"] == "PASSED":
                            continue
                        else:
                            return CheckResult.FAILED
                return CheckResult.PASSED

        return CheckResult.FAILED


check = RootContainers()



from __future__ import annotations

import asyncio
import json
import logging
import os.path
from collections.abc import Iterable
from pathlib import Path
from typing import Optional, Union, Dict, Any

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.platform_key import bridgecrew_dir
from checkov.common.bridgecrew.vulnerability_scanning.image_scanner import image_scanner, TWISTCLI_FILE_NAME
from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import \
    docker_image_scanning_integration
from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.output.report import Report, merge_reports
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.common import ImageDetails
from checkov.common.runners.base_runner import filter_ignored_paths, strtobool
from checkov.common.sca.commons import should_run_scan
from checkov.common.sca.output import add_to_report_sca_data, get_license_statuses
from checkov.common.util.file_utils import compress_file_gzip_base64
from checkov.common.util.dockerfile import is_docker_file
from checkov.common.util.http_utils import request_wrapper
from checkov.runner_filter import RunnerFilter
from checkov.sca_package.runner import Runner as PackageRunner


class Runner(PackageRunner):
    check_type = CheckType.SCA_IMAGE  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__(report_type=self.check_type)
        self._check_class: Optional[str] = None
        self._code_repo_path: Optional[Path] = None
        self._check_class = f"{image_scanner.__module__}.{image_scanner.__class__.__qualname__}"
        self.raw_report: Optional[Dict[str, Any]] = None
        self.base_url = bc_integration.api_url
        self.image_referencers: set[ImageReferencer] | None = None

    def should_scan_file(self, filename: str) -> bool:
        return is_docker_file(os.path.basename(filename))

    def scan(
            self,
            image_id: str,
            dockerfile_path: str,
            runner_filter: RunnerFilter | None = None,
    ) -> Dict[str, Any]:
        runner_filter = runner_filter or RunnerFilter()

        # skip complete run, if flag '--check' was used without a CVE check ID or the license policies
        if not should_run_scan(runner_filter.checks):
            return {}

        if not bc_integration.bc_api_key:
            logging.info("The --bc-api-key flag needs to be set to run SCA package scanning")
            return {}

        logging.info(f"SCA image scanning is scanning the image {image_id}")

        cached_results: Dict[str, Any] = image_scanner.get_scan_results_from_cache(image_id)
        if cached_results:
            logging.info(f"Found cached scan results of image {image_id}")
            return cached_results

        image_scanner.setup_scan(image_id, dockerfile_path, skip_extract_image_name=False)
        try:
            output_path = Path(f'results-{image_id}.json')
            scan_result = asyncio.run(self.execute_scan(image_id, output_path))
            self.upload_results_to_cache(output_path, image_id)
            logging.info(f"SCA image scanning successfully scanned the image {image_id}")
            return scan_result
        except Exception:
            raise

    async def execute_scan(
            self,
            image_id: str,
            output_path: Path,
    ) -> Dict[str, Any]:
        command = f"{Path(bridgecrew_dir) / TWISTCLI_FILE_NAME} images scan --address {docker_image_scanning_integration.get_proxy_address()} --token {docker_image_scanning_integration.get_bc_api_key()} --details --output-file \"{output_path}\" {image_id}"
        process = await asyncio.create_subprocess_shell(
            command, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
        )

        stdout, stderr = await process.communicate()

        # log output for debugging
        try:
            logging.debug(stdout.decode())
        except UnicodeDecodeError:
            logging.error("error was caught when trying to decode the \'stdout\' from twistcli.\n"
                          f"file content is:\n{image_scanner.dockerfile_content}.\n"
                          f"twistcli command is \'{command}\'", exc_info=True)

        exit_code = await process.wait()

        if exit_code:
            logging.error(stderr.decode())
            return {}

        # read the report file
        scan_result: Dict[str, Any] = json.loads(output_path.read_text())

        return scan_result

    def upload_results_to_cache(self, output_path: Path, image_id: str) -> None:
        image_id_sha = f"sha256:{image_id}" if not image_id.startswith("sha256:") else image_id

        request_body = {
            "compressedResult": compress_file_gzip_base64(str(output_path)),
            "compressionMethod": "gzip",
            "id": image_id_sha
        }
        response = request_wrapper(
            "POST", f"{self.base_url}/api/v1/vulnerabilities/scan-results",
            headers=bc_integration.get_default_headers("POST"), data=json.dumps(request_body)
        )

        if response.ok:
            logging.info(f"Successfully uploaded scan results to cache with id={image_id}")
        else:
            logging.info(f"Failed to upload scan results to cache with id={image_id}")

        output_path.unlink()

    def run(
            self,
            root_folder: str | Path | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
            **kwargs: str
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        self._code_repo_path = Path(root_folder) if root_folder else None

        if "dockerfile_path" in kwargs and "image_id" in kwargs:
            dockerfile_path = kwargs['dockerfile_path']
            image_id = kwargs['image_id']
            return self.get_image_id_report(dockerfile_path, image_id, runner_filter)
        report = Report(self.check_type)
        if not files and not root_folder:
            logging.debug("No resources to scan.")
            return report
        if files:
            self.pbar.initiate(len(files))
            # 'root_folder' should contain the common prefix so the absolute full path can be shortened later
            root_folder = os.path.split(os.path.commonprefix(files))[0]

            for file in files:
                self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(file, root_folder)})
                self.iterate_image_files(file, report, runner_filter)
                self.pbar.update()
            self.pbar.close()
        if root_folder:
            for root, d_names, f_names in os.walk(root_folder):
                filter_ignored_paths(root, d_names, runner_filter.excluded_paths, included_paths=self.included_paths())
                filter_ignored_paths(root, f_names, runner_filter.excluded_paths, included_paths=self.included_paths())
                for file in f_names:
                    abs_fname = os.path.join(root, file)
                    self.iterate_image_files(abs_fname, report, runner_filter, root_folder)
        return report

    def iterate_image_files(self, abs_fname: str, report: Report, runner_filter: RunnerFilter,
                            root_folder: Union[str, Path, None] = None) -> None:
        """
        Get workflow file, and get the list of images from every relevant imagereferencer, and create a unified vulnrability report
        :param root_folder: for CLI scans - the directory in which checkov runs
        :param abs_fname: file path to inspect
        :param report: unified report object
        :param runner_filter: filter for report
        """
        if not self.image_referencers:
            return
        for image_referencer in self.image_referencers:
            if image_referencer.is_workflow_file(abs_fname):
                images = image_referencer.get_images(file_path=abs_fname)
                for image in images:
                    image_cached_result: Dict[str, Any] = image_scanner.get_scan_results_from_cache(
                        f"image:{image.name}")

                    image_cached_report: dict[str, Any] = self.get_image_cached_results(dockerfile_path=abs_fname,
                                                                                        image=image,
                                                                                        image_cached_result=image_cached_result,
                                                                                        root_folder=root_folder)
                    if image_cached_report:
                        report.image_cached_results.append(image_cached_report)

                    if strtobool(os.getenv('CHECKOV_CREATE_SCA_IMAGE_REPORTS_FOR_IR', "True")):
                        image_report = self.get_image_report(dockerfile_path=abs_fname, image=image,
                                                             runner_filter=runner_filter,
                                                             image_cached_result=image_cached_result)
                        merge_reports(report, image_report)

    def get_report_from_scan_result(self, result: Dict[str, Any], dockerfile_path: str, rootless_file_path: str,
                                    image_details: ImageDetails | None, runner_filter: RunnerFilter) -> Report:
        report = Report(self.check_type)
        vulnerabilities = result.get("vulnerabilities", [])
        packages = result.get("packages", [])
        license_statuses = get_license_statuses(packages)

        add_to_report_sca_data(
            report=report,
            check_class=self._check_class,
            scanned_file_path=os.path.abspath(dockerfile_path),
            rootless_file_path=rootless_file_path,
            runner_filter=runner_filter,
            vulnerabilities=vulnerabilities,
            packages=packages,
            license_statuses=license_statuses,
            sca_details=image_details,
            report_type=self.report_type
        )
        return report

    def get_image_cached_results(self, dockerfile_path: str, image: Image, image_cached_result: Dict[str, Any],
                                 root_folder: Union[str, Path, None] = None) -> dict[str, Any]:
        """
            :param image_cached_result: twistcli result for image as saved in cache
            :param root_folder: for CLI scans - the directory in which checkov runs
            :param dockerfile_path: path of a file that might contain a container image
            :param image: Image object
            :return: cached_results report
        """

        if not image_cached_result:
            # TODO: do we want to trigger a scan in this case?
            logging.info(
                f"No cache hit for image {image.name} when getting cached results for dockerfile {dockerfile_path}")
            return {}
        payload: dict[str, Any] = docker_image_scanning_integration.create_report(
            twistcli_scan_result=image_cached_result,
            bc_platform_integration=bc_integration,
            file_path=dockerfile_path,
            file_content=f'image: {image.name}',
            docker_image_name=image.name,
            related_resource_id=image.related_resource_id,
            root_folder=root_folder)
        return payload

    def get_image_report(self, dockerfile_path: str, image: Image, runner_filter: RunnerFilter, image_cached_result: Dict[str, Any]) -> Report:
        """

        :param dockerfile_path: path of a file that might contain a container image
        :param image: Image object
        :param runner_filter:
        :return: vulnerability report
        """
        # skip complete run, if flag '--check' was used without a CVE check ID
        if not should_run_scan(runner_filter.checks):
            return Report(self.check_type)

        if image_cached_result:
            logging.info(f"Found cached scan results of image {image.name}")
            self.raw_report = image_cached_result
            result = image_cached_result.get('results', [{}])[0]
            image_id = self.extract_image_short_id(result)
            image_details = self.get_image_details_from_twistcli_result(scan_result=result, image_id=image_id,
                                                                        image_name=image.name,
                                                                        related_resource_id=image.related_resource_id)
            if self._code_repo_path:
                try:
                    dockerfile_path = str(Path(dockerfile_path).relative_to(self._code_repo_path))
                except ValueError:
                    # Path.is_relative_to() was implemented in Python 3.9
                    pass
            rootless_file_path = dockerfile_path.replace(Path(dockerfile_path).anchor, "", 1)
            rootless_file_path_to_report = f"{rootless_file_path} ({image.name} lines:{image.start_line}-" \
                                           f"{image.end_line} ({image_id}))"
            return self.get_report_from_scan_result(result, dockerfile_path, rootless_file_path_to_report,
                                                    image_details, runner_filter)

        elif strtobool(os.getenv("CHECKOV_EXPERIMENTAL_IMAGE_REFERENCING", "False")):
            # experimental flag on running image referencers via local twistcli
            image_id = ImageReferencer.inspect(image.name)
            if not image_id:
                logging.info(f"Unable to extract image id from {image.name}")
                return Report(self.check_type)
            scan_result = self.scan(image_id, dockerfile_path, runner_filter)
            if scan_result is None:
                return Report(self.check_type)

            self.raw_report = scan_result
            result = scan_result.get('results', [{}])[0]
            rootless_file_path_to_report = f"{dockerfile_path} ({image.name} lines:{image.start_line}-" \
                                           f"{image.end_line} ({image_id}))"
            return self.get_report_from_scan_result(result, dockerfile_path, rootless_file_path_to_report, None,
                                                    runner_filter)
        else:
            logging.info(f"No cache hit for image {image.name}")

        return Report(self.check_type)

    def get_image_id_report(self, dockerfile_path: str, image_id: str, runner_filter: RunnerFilter) -> Report:
        """
        THIS METHOD HANDLES CUSTOM IMAGE SCANNING THAT COMES DIRECTLY FROM CLI PARAMETERS
        """
        scan_result = self.scan(image_id, dockerfile_path, runner_filter)
        if scan_result is None:
            return Report(self.check_type)
        self.raw_report = scan_result
        result = scan_result.get('results', [{}])[0]
        image_details = self.get_image_details_from_twistcli_result(scan_result=result, image_id=image_id)
        if self._code_repo_path:
            try:
                dockerfile_path = str(Path(dockerfile_path).relative_to(self._code_repo_path))
            except ValueError:
                # Path.is_relative_to() was implemented in Python 3.9
                pass
        rootless_file_path = dockerfile_path.replace(Path(dockerfile_path).anchor, "", 1)
        rootless_file_path_to_report = f"{rootless_file_path} ({image_id})"
        return self.get_report_from_scan_result(result, dockerfile_path, rootless_file_path_to_report, image_details,
                                                runner_filter)

    def extract_image_short_id(self, scan_result: dict[str, Any]) -> str:
        """Extracts a shortened version of the image ID from the scan result"""

        if "id" not in scan_result:
            return "sha256:unknown"

        image_id: str = scan_result["id"]

        if image_id.startswith("sha256:"):
            return image_id[:17]
        return image_id[:10]

    def get_image_details_from_twistcli_result(self, scan_result: dict[str, Any], image_id: str,
                                               image_name: str | None = None,
                                               related_resource_id: str | None = None) -> ImageDetails:
        image_packages = scan_result.get('packages', [])
        image_package_types = {
            f'{package["name"]}@{package["version"]}': package['type']
            for package in image_packages
        }
        return ImageDetails(
            distro=scan_result.get('distro', ''),
            distro_release=scan_result.get('distroRelease', ''),
            package_types=image_package_types,
            image_id=image_id,
            name=image_name,
            related_resource_id=related_resource_id
        )

    def included_paths(self) -> Iterable[str]:
        return ['.github', '.circleci']



from __future__ import annotations

import json
import logging
from json import JSONDecodeError
from typing import TypeVar, overload, Any, Dict

import yaml

T = TypeVar("T")


@overload
def force_list(var: list[T]) -> list[T]:
    ...


@overload
def force_list(var: T) -> list[T]:
    ...


def force_list(var: T | list[T]) -> list[T]:
    if not isinstance(var, list):
        return [var]
    return var


def force_int(var: Any) -> int | None:
    try:
        if not isinstance(var, int):
            return int(var)
        return var
    except Exception:
        return None


def force_float(var: Any) -> float | None:
    try:
        if not isinstance(var, float):
            return float(var)
        return var
    except Exception:
        return None


def convert_str_to_bool(bool_str: bool | str) -> bool | str:
    if bool_str in ["true", '"true"', "True", '"True"']:
        return True
    elif bool_str in ["false", '"false"', "False", '"False"']:
        return False
    else:
        return bool_str


def force_dict(obj: Any) -> dict[str, Any] | None:
    """
    If the specified object is a dict, returns the object. If the specified object is a list or a tuple
    of length 1 or more, force_dict is called recursively on the first element. Else returns None.
    :param obj:
    :return:
    """
    if isinstance(obj, dict):
        return obj
    if (isinstance(obj, list) or isinstance(obj, tuple)) and len(obj) > 0:
        return force_dict(obj[0])
    return None


def is_json(data: str) -> bool:
    try:
        parsed = json.loads(data)
        return isinstance(parsed, dict)
    except (TypeError, ValueError):
        logging.debug(f"could not parse json data: {data}")
        return False


def is_yaml(data: str) -> bool:
    try:
        parsed = yaml.safe_load(data)
        return isinstance(parsed, dict)
    except yaml.YAMLError:
        logging.debug(f"could not parse yaml data: {data}")
        return False


def extract_policy_dict(policy: Any) -> dict[str, Any] | None:
    if isinstance(policy, dict):
        return policy
    if isinstance(policy, str):
        try:
            policy_dict: dict[str, Any] = json.loads(policy)
            return policy_dict
        except JSONDecodeError:
            return None

    return None


def extract_json(json_str: Any) -> dict[str, Any] | list[dict[str, Any]] | None:
    """Tries to return a json object from a possible string value"""

    if isinstance(json_str, list):
        return json_str

    return extract_policy_dict(json_str)


def convert_csv_string_arg_to_list(csv_string_arg: list[str] | str | None) -> list[str]:
    """
    Converts list type arguments that also support comma delimited strings into a list.
    For instance the --check flag in the CLI:
        checkov -c CKV_1,CKV2
        will translate to ['CKV_1', 'CKV_2']
    :param csv_string_arg: Comma delimited string
    :return: List of strings or empty list
    """
    if csv_string_arg is None:
        return []
    if isinstance(csv_string_arg, str):
        return csv_string_arg.split(',')
    elif isinstance(csv_string_arg, list) and len(csv_string_arg) == 1:
        return csv_string_arg[0].split(',')
    else:
        return csv_string_arg


def convert_prisma_policy_filter_to_dict(filter_string: str) -> Dict[Any, Any]:
    """
    Converts the filter string to a dict. For example:
    'policy.label=label,cloud.type=aws' becomes -->
    {'policy.label': 'label1', 'cloud.type': 'aws'}
    Note that the API does not accept lists https://prisma.pan.dev/api/cloud/cspm/policy#operation/get-policies-v2
    This is not allowed: policy.label=label1,label2
    """
    filter_params = {}
    if isinstance(filter_string, str) and filter_string:
        filter_string = "".join(filter_string.split())
        try:
            for f in filter_string.split(','):
                f_name, f_value = f.split('=')
                filter_params[f_name] = f_value
        except (IndexError, ValueError) as e:
            logging.debug(f"Invalid filter format: {e}")
    return filter_params



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.common.util.data_structures_utils import find_in_dict
from checkov.common.util.type_forcers import force_list
from checkov.terraform.image_referencer.base_provider import BaseTerraformProvider

if TYPE_CHECKING:
    from networkx import DiGraph


class AzureTerraformProvider(BaseTerraformProvider):
    def __init__(self, graph_connector: DiGraph) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_AZURE_IMAGE_RESOURCE_TYPES,
        )


def extract_images_from_azurerm_batch_pool(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    containers = find_in_dict(input_dict=resource, key_path="container_configuration/container_image_names")
    if isinstance(containers, list):
        image_names.extend(container for container in containers if isinstance(container, str))

    return image_names


def extract_images_from_azurerm_container_group(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    containers = resource.get("container")
    if containers:
        for container in force_list(containers):
            name = container.get("image")
            if name and isinstance(name, str):
                image_names.append(name)
    containers = resource.get("init_container")
    if containers:
        for container in force_list(containers):
            name = container.get("image")
            if name and isinstance(name, str):
                image_names.append(name)

    return image_names


def extract_images_from_azurerm_linux_function_app(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    docker = find_in_dict(input_dict=resource, key_path="site_config/application_stack/docker")
    if isinstance(docker, dict):
        name = docker.get("image_name")
        tag = docker.get("image_tag")
        if name and isinstance(name, str) and tag and isinstance(tag, str):
            image_names.append(f"{name}:{tag}")

    return image_names


def extract_images_from_azurerm_linux_web_app(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    app_stack = find_in_dict(input_dict=resource, key_path="site_config/application_stack")
    if isinstance(app_stack, dict):
        name = app_stack.get("docker_image")
        tag = app_stack.get("docker_image_tag")
        if name and isinstance(name, str) and tag and isinstance(tag, str):
            image_names.append(f"{name}:{tag}")

    return image_names


def extract_images_from_azurerm_spring_cloud_container_deployment(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    name = resource.get("image")
    if name and isinstance(name, str):
        image_names.append(name)

    return image_names


def extract_images_from_azurerm_windows_web_app(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    app_stack = find_in_dict(input_dict=resource, key_path="site_config/application_stack")
    if isinstance(app_stack, dict):
        name = app_stack.get("docker_container_name")
        tag = app_stack.get("docker_container_tag")
        if name and isinstance(name, str) and tag and isinstance(tag, str):
            image_names.append(f"{name}:{tag}")

    return image_names


# needs to be at the bottom to add the defined functions
SUPPORTED_AZURE_IMAGE_RESOURCE_TYPES = {
    "azurerm_batch_pool": extract_images_from_azurerm_batch_pool,
    "azurerm_container_group": extract_images_from_azurerm_container_group,
    "azurerm_linux_function_app": extract_images_from_azurerm_linux_function_app,
    "azurerm_linux_function_app_slot": extract_images_from_azurerm_linux_function_app,
    "azurerm_linux_web_app": extract_images_from_azurerm_linux_web_app,
    "azurerm_linux_web_app_slot": extract_images_from_azurerm_linux_web_app,
    "azurerm_spring_cloud_container_deployment": extract_images_from_azurerm_spring_cloud_container_deployment,
    "azurerm_windows_web_app": extract_images_from_azurerm_windows_web_app,
    "azurerm_windows_web_app_slot": extract_images_from_azurerm_windows_web_app,
}



from checkov.terraform.checks.resource.ncp.NACLInboundCheck import NACLInboundCheck


class NACLInbound22(NACLInboundCheck):
    def __init__(self) -> None:
        super().__init__(check_id="CKV_NCP_10", port=22)


check = NACLInbound22()



from __future__ import annotations

from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check
from checkov.common.models.enums import CheckCategories, CheckResult
from typing import Dict, Any, List, TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Iterable


class RbacOperation():
    """
    A collection of RBAC permissions that permit a certain operation within Kubernetes.
    For example, the RbacOperation below denotes a write operation on admission webhooks.
    control_webhooks = RbacOperation(
        apigroups=["admissionregistration.k8s.io"],
        verbs=["create", "update", "patch"],
        resources=["mutatingwebhookconfigurations", "validatingwebhookconfigurations"])
    Rules matching an apiGroup, verb and resource should be able to perform the operation.
    """
    __slots__ = ("apigroups", "resources", "verbs")

    def __init__(self, apigroups: List[str], verbs: List[str], resources: List[str]) -> None:
        self.apigroups = apigroups
        self.verbs = verbs
        self.resources = resources


class BaseRbacK8sCheck(BaseK8Check):
    """
    Base class for checks that evaluate RBAC permissions in Roles and ClusterRoles
    """
    def __init__(self, name: str, id: str, supported_entities: Iterable[str] | None = None) -> None:
        if supported_entities is None:
            supported_entities = ("Role", "ClusterRole")
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities)
        # A role that grants *ALL* the RbacOperation in failing_operations fails this check
        self.failing_operations: list[RbacOperation] = []

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        rules = conf.get("rules")
        if rules and isinstance(rules, list):
            for operation in self.failing_operations:
                # if one operation can't be found, check passes
                if not any(self.rule_can(rule, operation) for rule in rules):
                    return CheckResult.PASSED
            # all operations were found, therefore the check fails
            return CheckResult.FAILED
        
        return CheckResult.PASSED

    # Check if a rule has an apigroup, verb, and resource specified in @operation
    def rule_can(self, rule: Dict[str, Any], operation: RbacOperation) -> bool:
        return self.apigroup_or_wildcard(rule, operation.apigroups) and \
            self.verb_or_wildcard(rule, operation.verbs) and \
            self.resource_or_wildcard(rule, operation.resources)

    def apigroup_or_wildcard(self, rule: Dict[str, Any], apigroups: List[str]) -> bool:
        return self.value_or_wildcard(rule, "apiGroups", apigroups)

    def verb_or_wildcard(self, rule: Dict[str, Any], verbs: List[str]) -> bool:
        return self.value_or_wildcard(rule, "verbs", verbs)

    def resource_or_wildcard(self, rule: Dict[str, Any], resources: List[str]) -> bool:
        if "resources" in rule:
            for granted_resource in rule["resources"]:
                if self.is_wildcard(granted_resource):
                    return True
                for failing_resource in resources:
                    if granted_resource == failing_resource:
                        return True
                    # Check for '*/subresource' syntax
                    if "/" in failing_resource and "/" in granted_resource:
                        if granted_resource == "*/" + failing_resource.split("/")[1]:
                            return True
        return False

    # Check if rule has a key with a wildcard or a value from @value_list
    def value_or_wildcard(self, rule: Dict[str, Any], key: str, value_list: List[str]) -> bool:
        if key in rule:
            for value in rule[key]:
                if self.is_wildcard(value) or value in value_list:
                    return True
        return False

    # Check if value is a K8s RBAC wildcard
    def is_wildcard(self, value: str) -> bool:
        return value == "*"



from __future__ import annotations

from checkov.github.base_github_branch_security import BranchSecurity


class GithubBranchDisallowDeletions(BranchSecurity):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection rules does not allow deletions - CIS 1.1.17"
        id = "CKV_GITHUB_18"
        super().__init__(
            name=name,
            id=id
        )

    def get_evaluated_keys(self) -> list[str]:
        return ['allow_deletions/enabled']

    def get_expected_value(self) -> bool:
        return False


check = GithubBranchDisallowDeletions()



from __future__ import annotations
from typing import Any
from checkov.common.models.enums import CheckResult

from checkov.gitlab_ci.checks.base_gitlab_ci_check import BaseGitlabCICheck
from checkov.yaml_doc.enums import BlockType


class SuspectCurlInScript(BaseGitlabCICheck):
    def __init__(self) -> None:
        name = "Suspicious use of curl with CI environment variables in script"
        id = "CKV_GITLABCI_1"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=('*.script[]',)
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        for line in conf.values():
            if not isinstance(line, str):
                continue
            if line.startswith("curl") and "$CI" in line:
                return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf


check = SuspectCurlInScript()



from __future__ import annotations

import json
import uuid

import requests
import logging
import time
import os
import aiohttp
import asyncio
from typing import Any, TYPE_CHECKING, cast, Optional, overload

from urllib3.response import HTTPResponse

from checkov.common.bridgecrew.bc_source import SourceType
from checkov.common.util.consts import DEV_API_GET_HEADERS, DEV_API_POST_HEADERS, PRISMA_API_GET_HEADERS, \
    PRISMA_PLATFORM, BRIDGECREW_PLATFORM
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.version import version as checkov_version

if TYPE_CHECKING:
    from requests import Response

logger = logging.getLogger(__name__)


@overload
def normalize_bc_url(url: None) -> None:
    ...


@overload
def normalize_bc_url(url: str) -> str:
    ...


def normalize_bc_url(url: str | None) -> str | None:
    if not url:
        return None
    return url.lower().replace('http:', 'https:').strip().rstrip('/')


def normalize_prisma_url(url: str | None) -> str | None:
    """ Correct common Prisma Cloud API URL misconfigurations """
    if not url:
        return None
    return url.lower().replace('//app', '//api').replace('http:', 'https:').strip().rstrip('/')


def get_auth_error_message(status: int, is_prisma: bool, is_s3_upload: bool) -> str:
    platform_type = PRISMA_PLATFORM if is_prisma else BRIDGECREW_PLATFORM
    error_message = f'Received unexpected response from platform (status code {status}). Please verify ' \
                    f'that your API token is valid and has permissions to call the {platform_type} APIs.'
    if platform_type == PRISMA_PLATFORM:
        error_message += 'The key must be associated with a Developer or Sys Admin role / permission group.'
    elif is_s3_upload:
        # This part only applies to S3 upload, but not downloading the run config
        error_message += 'The key must be associated with any role besides Auditor.'
    return error_message


def extract_error_message(response: requests.Response | HTTPResponse) -> Optional[str]:
    if (isinstance(response, requests.Response) and response.content) or (
            isinstance(response, HTTPResponse) and response.data):
        raw = response.content if isinstance(response, requests.Response) else response.data
        try:
            content = json.loads(raw)
            if 'message' in content:
                return cast(str, content['message'])
            elif 'Message' in content:
                return cast(str, content['Message'])
        except Exception:
            logging.debug(f'Failed to parse the response content: {raw.decode()}')

    return response.reason


def get_auth_header(token: str) -> dict[str, str]:
    return {
        'Authorization': token
    }


def get_prisma_auth_header(token: str) -> dict[str, str]:
    return {
        'x-redlock-auth': token
    }


def get_version_headers(client: str, client_version: str | None) -> dict[str, str]:
    return {
        'x-api-client': client,
        'x-api-version': client_version or "unknown",
        'x-api-checkov-version': checkov_version
    }


def get_user_agent_header() -> dict[str, str]:
    return {'User-Agent': f'checkov/{checkov_version}'}


def get_default_get_headers(client: SourceType, client_version: str | None) -> dict[str, Any]:
    return merge_dicts(DEV_API_GET_HEADERS, get_version_headers(client.name, client_version), get_user_agent_header())


def get_default_post_headers(client: SourceType, client_version: str | None) -> dict[str, Any]:
    return merge_dicts(DEV_API_POST_HEADERS, get_version_headers(client.name, client_version), get_user_agent_header())


def get_prisma_get_headers() -> dict[str, str]:
    return merge_dicts(PRISMA_API_GET_HEADERS, get_user_agent_header())


def request_wrapper(
        method: str,
        url: str,
        headers: dict[str, Any],
        data: Any | None = None,
        json: dict[str, Any] | None = None,
        should_call_raise_for_status: bool = False,
        params: dict[str, Any] | None = None
) -> Response:
    # using of "retry" mechanism for 'requests.request' due to unpredictable 'ConnectionError' and 'HttpError'
    # instances that appears from time to time.
    # 'ConnectionError' instances that appeared:
    # * 'Connection aborted.', ConnectionResetError(104, 'Connection reset by peer').
    # * 'Connection aborted.', OSError(107, 'Socket not connected').
    # 'HTTPError' instances that appeared:
    # * 403 Client Error: Forbidden for url.
    # * 504 Server Error: Gateway Time-out for url.

    request_max_tries = int(os.getenv('REQUEST_MAX_TRIES', 3))
    sleep_between_request_tries = float(os.getenv('SLEEP_BETWEEN_REQUEST_TRIES', 1))

    for i in range(request_max_tries):
        try:
            headers["X-Request-Id"] = str(uuid.uuid4())
            response = requests.request(method, url, headers=headers, data=data, json=json, params=params)
            if should_call_raise_for_status:
                response.raise_for_status()
            return response
        except requests.exceptions.ConnectionError as connection_error:
            logging.error(f"Connection error on request {method}:{url},\ndata:\n{data}\njson:{json}\nheaders:{headers}")
            if i != request_max_tries - 1:
                sleep_secs = sleep_between_request_tries * (i + 1)
                logging.info(f"retrying attempt number {i + 2} in {sleep_secs} seconds")
                time.sleep(sleep_secs)
                continue

            logging.exception("request_wrapper connection error")
            raise connection_error
        except requests.exceptions.HTTPError as http_error:
            status_code = http_error.response.status_code
            logging.error(f"HTTP error on request {method}:{url},\ndata:\n{data}\njson:{json}\nheaders:{headers}")
            if (status_code >= 500 or status_code == 403) and i != request_max_tries - 1:
                sleep_secs = sleep_between_request_tries * (i + 1)
                logging.info(f"retrying attempt number {i + 2} in {sleep_secs} seconds")
                time.sleep(sleep_secs)
                continue

            logging.exception("request_wrapper http error")
            raise http_error
    else:
        raise Exception("Unexpected behavior: the method \'request_wrapper\' should be terminated inside the above for-"
                        "loop")


async def aiohttp_client_session_wrapper(
        url: str,
        headers: dict[str, Any],
        payload: dict[str, Any]
) -> int:
    request_max_tries = int(os.getenv('REQUEST_MAX_TRIES', 3))
    sleep_between_request_tries = float(os.getenv('SLEEP_BETWEEN_REQUEST_TRIES', 1))

    try:  # TODO: test again, when Python 3.11 is out
        import aiodns  # type: ignore[import]  # noqa: F401
        resolver: "aiohttp.abc.AbstractResolver" = aiohttp.AsyncResolver()
    except ImportError:
        resolver = aiohttp.ThreadedResolver()

    # adding retry mechanism for avoiding the next repeated unexpected issues:
    # 1. Gateway Timeout from the server
    # 2. ClientOSError
    async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(resolver=resolver)) as session:
        for i in range(request_max_tries):
            logging.info(
                f"[http_utils](aiohttp_client_session_wrapper) reporting attempt {i + 1} out of {request_max_tries}")
            try:
                async with session.post(
                        url=url, headers=headers, json=payload
                ) as response:
                    content = await response.text()
                if response.ok:
                    logging.info(f"[http_utils](aiohttp_client_session_wrapper) - done successfully to url: \'{url}\'")
                    return 0
                elif i != request_max_tries - 1:
                    await asyncio.sleep(sleep_between_request_tries * (i + 1))
                    continue
                else:
                    logging.error(f"[http_utils](aiohttp_client_session_wrapper) - Failed to send report to "
                                  f"url \'{url}\'")
                    logging.error(f"Status code: {response.status}, Reason: {response.reason}, Content: {content}")
                    return 1
            except aiohttp.ClientOSError:
                if i != request_max_tries - 1:
                    await asyncio.sleep(sleep_between_request_tries * (i + 1))
                    continue
                else:
                    logging.error(f"[http_utils](aiohttp_client_session_wrapper) - ClientOSError when sending report "
                                  f"to url: \'{url}\'")
                    raise
            except Exception as e:
                logging.error(f"[http_utils](aiohttp_client_session_wrapper) - exception when sending report "
                              f"to url: \'{url}\':\n\'{e}\'")
                raise

        else:
            raise Exception("Unexpected behavior: the method \'aiohttp_client_session_wrapper\' should be terminated "
                            "inside the above for-loop")



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceHttps20Enabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that 'HTTP Version' is the latest if used to run the web app"
        id = "CKV_AZURE_18"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'site_config/[0]/http2_enabled'


check = AppServiceHttps20Enabled()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ECSTaskDefinitionRoleCheck(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that the Execution Role ARN and the Task Role ARN are different in ECS Task definitions"
        id = "CKV_AWS_249"
        supported_resources = ("aws_ecs_task_definition",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["execution_role_arn", "task_role_arn"]
        if "execution_role_arn" in conf.keys() and "task_role_arn" in conf.keys():
            if conf["execution_role_arn"] == conf["task_role_arn"]:
                return CheckResult.FAILED
        return CheckResult.PASSED


check = ECSTaskDefinitionRoleCheck()



from __future__ import annotations

from typing import Iterable

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories

from checkov.github.registry import registry


class BaseGithubCheck(BaseCheck):
    def __init__(self, name: str, id: str, categories: Iterable[CheckCategories], supported_entities: Iterable[str],
                 block_type: str, path: str | None = None, guideline: str | None = None) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
            guideline=guideline,
        )
        self.path = path
        registry.register(self)



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class LBListenerUsingHTTPS(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure Load Balancer Listener Using HTTPS"
        id = "CKV_NCP_24"
        supported_resources = ("ncloud_lb_listener",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'protocol'

    def get_expected_value(self):
        return 'HTTPS'


check = LBListenerUsingHTTPS()



import os

from checkov.common.bridgecrew.run_metadata.abstract_run_metadata_extractor import RunMetaDataExtractor


class GithubActionsRunMetadataExtractor(RunMetaDataExtractor):
    def is_current_ci(self) -> bool:
        if os.getenv("GITHUB_ACTIONS", ""):
            return True
        return False

    def __init__(self) -> None:
        server_url = os.getenv('GITHUB_SERVER_URL', '')
        from_branch = os.getenv('GIT_BRANCH', "master")
        to_branch = os.getenv('GITHUB_BASE_REF', "")
        pr_id = os.getenv("$GITHUB_REF", "//").split("/")[2]
        repository = os.getenv('GITHUB_REPOSITORY', "")
        pr_url = f"{server_url}/{repository}/pull/{pr_id}"
        commit_hash = os.getenv("GITHUB_SHA", "")
        commit_url = f"{server_url}/{repository}/commit/${commit_hash}"
        author_name = os.getenv("GITHUB_ACTOR", "")
        author_url = f"{server_url}/{author_name}"
        run_id = os.getenv("GITHUB_RUN_NUMBER", "")
        run_url = f"{server_url}/{repository}/actions/runs/{run_id}"
        repository_url = f"{server_url}/{repository}"

        super().__init__(from_branch=from_branch,
                         to_branch=to_branch,
                         pr_id=pr_id,
                         pr_url=pr_url,
                         commit_hash=commit_hash,
                         commit_url=commit_url,
                         author_name=author_name,
                         author_url=author_url,
                         run_id=run_id,
                         run_url=run_url,
                         repository_url=repository_url)


extractor = GithubActionsRunMetadataExtractor()



from __future__ import annotations

import os
from typing import TYPE_CHECKING

from checkov.common.graph.db_connectors.db_connector import DBConnector
from checkov.common.graph.graph_manager import GraphManager
from checkov.dockerfile.graph_builder.local_graph import DockerfileLocalGraph
from checkov.dockerfile.utils import get_scannable_file_paths, get_files_definitions

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction  # only in extra_stubs
    from networkx import DiGraph


class DockerfileGraphManager(GraphManager[DockerfileLocalGraph, "dict[str, dict[str, list[_Instruction]]]"]):
    def __init__(self, db_connector: DBConnector[DiGraph], source: str = "Dockerfile") -> None:
        super().__init__(db_connector=db_connector, parser=None, source=source)

    def build_graph_from_source_directory(
        self,
        source_dir: str,
        local_graph_class: type[DockerfileLocalGraph] = DockerfileLocalGraph,
        render_variables: bool = True,
        parsing_errors: dict[str, Exception] | None = None,
        download_external_modules: bool = False,
        excluded_paths: list[str] | None = None,
    ) -> tuple[DockerfileLocalGraph, dict[str, dict[str, list[_Instruction]]]]:
        file_paths = get_scannable_file_paths(root_folder=source_dir, excluded_paths=excluded_paths)
        filepath_fn = lambda f: f"/{os.path.relpath(f, os.path.commonprefix((source_dir, f)))}"
        definitions, _ = get_files_definitions(files=file_paths, filepath_fn=filepath_fn)

        local_graph = self.build_graph_from_definitions(definitions=definitions)

        return local_graph, definitions

    def build_graph_from_definitions(
        self,
        definitions: dict[str, dict[str, list[_Instruction]]],
        render_variables: bool = False,
    ) -> DockerfileLocalGraph:
        local_graph = DockerfileLocalGraph(definitions=definitions)
        local_graph.build_graph(render_variables=render_variables)

        return local_graph



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any

from checkov.common.models.consts import ANY_VALUE


class DropletSSHKeys(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure the droplet specifies an SSH key"
        id = "CKV_DIO_2"
        supported_resources = ['digitalocean_droplet']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "ssh_keys"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = DropletSSHKeys()



from __future__ import annotations

from typing import Union, List, Dict, Any, TYPE_CHECKING

from checkov.terraform.checks.data.BaseCloudsplainingIAMCheck import BaseCloudsplainingIAMCheck

if TYPE_CHECKING:
    from cloudsplaining.scan.policy_document import PolicyDocument


class CloudSplainingWriteAccess(BaseCloudsplainingIAMCheck):
    def __init__(self) -> None:
        name = "Ensure IAM policies does not allow write access without constraints"
        id = "CKV_AWS_111"
        super().__init__(name=name, id=id)

    def cloudsplaining_analysis(self, policy: PolicyDocument) -> Union[List[str], List[Dict[str, Any]]]:
        return policy.write_actions_without_constraints


check = CloudSplainingWriteAccess()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AthenaWorkgroupConfiguration(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure Athena Workgroup should enforce configuration to prevent client disabling encryption"
        id = "CKV_AWS_82"
        supported_resources = ['aws_athena_workgroup']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "configuration/[0]/enforce_workgroup_configuration"


check = AthenaWorkgroupConfiguration()



from __future__ import annotations

from checkov.github.base_github_branch_security import BranchSecurity


class GithubBranchRequireStatusChecks(BranchSecurity):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection requires status checks - CIS 1.1.9"
        id = "CKV_GITHUB_14"
        super().__init__(name=name, id=id)

    def get_evaluated_keys(self) -> list[str]:
        return ["required_status_checks"]


check = GithubBranchRequireStatusChecks()



from checkov.terraform.checks.resource.aws.AbsNACLUnrestrictedIngress import AbsNACLUnrestrictedIngress


class NACLUnrestrictedIngress3389(AbsNACLUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_AWS_231", port=3389)


check = NACLUnrestrictedIngress3389()



from __future__ import annotations

from abc import abstractmethod

from jsonpath_ng import parse
from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.org_security import schema as org_security_schema
from checkov.json_doc.enums import BlockType


class OrgSecurity(BaseGithubCheck):
    def __init__(self, id: str, name: str) -> None:
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult | None:  # type:ignore[override]
        if org_security_schema.validate(conf):
            jsonpath_expression = parse("$..{}".format(self.get_evaluated_keys()[0].replace("/", ".")))
            if all(match.value == self.get_expected_value() for match in jsonpath_expression.find(conf)):
                return CheckResult.PASSED
            else:
                return CheckResult.FAILED
        return None

    def get_expected_value(self) -> int | bool | str:
        return True

    @abstractmethod
    def get_evaluated_keys(self) -> list[str]:
        pass



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class GKEKubernetesRBACGoogleGroups(BaseResourceValueCheck):
    def __init__(self):
        name = "Manage Kubernetes RBAC users with Google Groups for GKE"
        id = "CKV_GCP_65"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'authenticator_groups_config/[0]/security_group'

    def get_expected_values(self):
        return [ANY_VALUE]


check = GKEKubernetesRBACGoogleGroups()



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Callable, Dict
from typing_extensions import TypeAlias, TypedDict

if TYPE_CHECKING:
    from checkov.common.bridgecrew.severities import Severity
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.models.enums import CheckResult


_ScannerCallableAlias: TypeAlias = Callable[
    [str, "BaseCheck", "_SkippedCheck", "dict[str, Any]", str, str, "dict[str, Any]"], None
]

_Resource: TypeAlias = str
_Attribute: TypeAlias = str
ResourceAttributesToOmit: TypeAlias = Dict[_Resource, _Attribute]


class _CheckResult(TypedDict, total=False):
    result: "CheckResult" | tuple["CheckResult", dict[str, Any]]
    suppress_comment: str
    evaluated_keys: list[str]
    results_configuration: dict[str, Any] | None
    check: BaseCheck
    entity: dict[str, Any]  # only exists for graph results


class _SkippedCheck(TypedDict, total=False):
    bc_id: str | None
    id: str
    suppress_comment: str
    line_number: int | None


class _BaselineFinding(TypedDict):
    resource: str
    check_ids: list[str]


class _BaselineFailedChecks(TypedDict):
    file: str
    findings: list[_BaselineFinding]


class _ReducedScanReport(TypedDict):
    checks: _ReducedScanReportCheck
    image_cached_results: list[dict[str, Any]]


class _ReducedScanReportCheck(TypedDict):
    failed_checks: list[dict[str, Any]]
    passed_checks: list[dict[str, Any]]
    skipped_checks: list[dict[str, Any]]


class _CicdDetails(TypedDict, total=False):
    commit: str | None
    pr: str | None
    runId: str | None
    scaCliScanId: str | None


class _ExitCodeThresholds(TypedDict):
    soft_fail: bool
    soft_fail_checks: list[str]
    soft_fail_threshold: Severity | None
    hard_fail_checks: list[str]
    hard_fail_threshold: Severity | None


class _LicenseStatus(TypedDict):
    package_name: str
    package_version: str
    policy: str
    license: str
    status: str



import os

DEFAULT_EXTERNAL_MODULES_DIR = ".external_modules"
RESOLVED_MODULE_ENTRY_NAME = "__resolved__"
START_LINE = '__startline__'
END_LINE = '__endline__'

DEV_API_GET_HEADERS = {
    'Accept': 'application/json'
}

DEV_API_POST_HEADERS = {
    'Accept': 'application/json',
    'Content-Type': 'application/json'
}

PRISMA_API_GET_HEADERS = {
    'Accept': 'application/json; charset=UTF-8'
}

PARSE_ERROR_FAIL_FLAG = 'CKV_PARSE_ERROR_FAIL'

PRISMA_PLATFORM = 'Prisma Cloud Code Security'
BRIDGECREW_PLATFORM = 'Bridgecrew'

MAX_IAC_FILE_SIZE = int(os.getenv('CHECKOV_MAX_IAC_FILE_SIZE', '50_000_000'))  # 50 MB is default limit

CHECKOV_RUN_SCA_PACKAGE_SCAN_V2 = os.getenv('CHECKOV_RUN_SCA_PACKAGE_SCAN_V2', '').lower() == 'true'



from checkov.terraform.checks.resource.azure.NSGRulePortAccessRestricted import NSGRulePortAccessRestricted


class NSGRuleRDPAccessRestricted(NSGRulePortAccessRestricted):
    def __init__(self) -> None:
        super().__init__(
            name="Ensure that RDP access is restricted from the internet",
            check_id="CKV_AZURE_9",
            port=3389,
        )


check = NSGRuleRDPAccessRestricted()



from __future__ import annotations

import re
from typing import TYPE_CHECKING

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction

ISABSOLUTE = re.compile('^"?((/[A-Za-z0-9-_+]*)|([A-Za-z0-9-_+]:\\\\.*)|(\\$[{}A-Za-z0-9-_+].*))')


class WorkdirIsAbsolute(BaseDockerfileCheck):
    def __init__(self) -> None:
        """
        For clarity and reliability, you should always use absolute paths for your WORKDIR.
        """
        name = "Ensure that WORKDIR values are absolute paths"
        id = "CKV_DOCKER_10"
        supported_instructions = ("WORKDIR",)
        categories = (CheckCategories.CONVENTION,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_resource_conf(self, conf: list[_Instruction]) -> tuple[CheckResult, list[_Instruction] | None]:
        workdirs = []
        for workdir in conf:
            path = workdir["value"]
            if isinstance(path, str) and not re.match(ISABSOLUTE, path):
                workdirs.append(workdir)

        if workdirs:
            return CheckResult.FAILED, workdirs

        return CheckResult.PASSED, None


check = WorkdirIsAbsolute()



from __future__ import annotations

import re
from typing import TYPE_CHECKING


if TYPE_CHECKING:
    from detect_secrets.util.code_snippet import CodeSnippet

from checkov.secrets.parsers.multiline_parser import BaseMultiLineParser


START_OBJ_END_OF_LINE = r'({\s*}?\s*,?\s*$)'
START_OBJ_START_OF_LINE = r'(^\s*{)'

START_OBJ = re.compile(fr'{START_OBJ_START_OF_LINE}|{START_OBJ_END_OF_LINE}')
START_OBJ_INCLUDING_CURRENT_LINE = re.compile(START_OBJ_START_OF_LINE)
END_OBJ = re.compile(r'(^\s*\}\s*)|({?\s*\}\s*,?\s*$)')
WHOLE_OBJ_INLINE = re.compile(r'{[^{]*{?\s*\}\s*,?\s*$')


class JsonMultiLineParser(BaseMultiLineParser):

    def consecutive_lines_in_same_object(
        self,
        raw_context: CodeSnippet | None,
        other_line_idx: int,
    ) -> bool:
        if not raw_context or not 0 <= other_line_idx < len(raw_context.lines):
            return False
        higher_line = raw_context.lines[other_line_idx]
        lower_line = raw_context.target_line
        if other_line_idx > raw_context.target_index:
            higher_line, lower_line = lower_line, higher_line
        if self.is_object_end(higher_line) or re.search(START_OBJ_INCLUDING_CURRENT_LINE, lower_line):
            return False
        if self.is_object_start(higher_line):
            return bool(re.search(START_OBJ_INCLUDING_CURRENT_LINE, higher_line))
        return True

    @staticmethod
    def is_object_start(
        line: str
    ) -> bool:
        return bool(re.search(START_OBJ, line) and not re.search(WHOLE_OBJ_INLINE, line))

    @staticmethod
    def is_object_end(
        line: str
    ) -> bool:
        return bool(re.search(END_OBJ, line) and not re.search(WHOLE_OBJ_INLINE, line))

    @staticmethod
    def is_line_comment(
        line: str
    ) -> bool:
        return False


json_multiline_parser = JsonMultiLineParser()



from __future__ import annotations

from typing import Any

from checkov.common.images.image_referencer import Image
from checkov.gitlab_ci.common.resource_id_utils import generate_resource_key_recursive


class GitlabCiProvider:
    __slots__ = ("supported_keys", "workflow_config", "file_path")

    def __init__(self, workflow_config: dict[str, Any], file_path: str):
        self.supported_keys = ("image", "services")
        self.workflow_config = workflow_config
        self.file_path = file_path

    @staticmethod
    def _get_start_end_lines(entity: dict[str, Any]) -> tuple[int, int]:
        return entity.get('__startline__', 0), entity.get('__endline__', 0)

    def extract_images_from_workflow(self) -> list[Image]:
        images = []
        for job_object in self.workflow_config.values():
            if isinstance(job_object, dict):
                start_line, end_line = GitlabCiProvider._get_start_end_lines(job_object)
                for key, subjob in job_object.items():
                    if key in self.supported_keys:
                        image_name = ""
                        if isinstance(subjob, dict):
                            start_line, end_line = GitlabCiProvider._get_start_end_lines(subjob)
                            image_name = subjob['name']
                        elif isinstance(subjob, str):
                            image_name = subjob
                        elif isinstance(subjob, list):
                            for service in subjob:
                                if isinstance(service, dict):
                                    start_line, end_line = GitlabCiProvider._get_start_end_lines(service)
                                    image_name = service['name']
                                elif isinstance(service, str):
                                    image_name = service
                                if image_name:
                                    image_obj = Image(
                                        file_path=self.file_path,
                                        name=image_name,
                                        start_line=start_line,
                                        end_line=end_line,
                                        related_resource_id=generate_resource_key_recursive(conf=self.workflow_config,
                                                                                            key='',
                                                                                            start_line=start_line,
                                                                                            end_line=end_line)
                                    )
                                    images.append(image_obj)
                                    image_name = ""
                        if image_name:
                            image_obj = Image(
                                file_path=self.file_path,
                                name=image_name,
                                start_line=start_line,
                                end_line=end_line,
                                related_resource_id=generate_resource_key_recursive(conf=self.workflow_config,
                                                                                    key='', start_line=start_line,
                                                                                    end_line=end_line)
                            )
                            images.append(image_obj)
        return images



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GKEBinaryAuthorization(BaseResourceCheck):
    def __init__(self):
        name = "Ensure use of Binary Authorization"
        id = "CKV_GCP_66"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'binary_authorization' in conf.keys():
            binary_authorization = conf["binary_authorization"][0]
            if isinstance(binary_authorization, dict) and 'evaluation_mode' in binary_authorization:
                # Google provider version >= v4.31.0
                if binary_authorization.get("evaluation_mode") == ["PROJECT_SINGLETON_POLICY_ENFORCE"]:
                    return CheckResult.PASSED
                # Google provider version v4.29.0 and v4.30.0
                elif binary_authorization.get("evaluation_mode") == [True]:
                    return CheckResult.PASSED
        # Google provider version <= v4.28.0
        if conf.get("enable_binary_authorization") == [True]:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = GKEBinaryAuthorization()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class GithubRepositoryVulnerabilityAlerts(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure GitHub repository has vulnerability alerts enabled"
        id = "CKV_GIT_3"
        supported_resources = ["github_repository"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # GitHub disables the alerts when archiving the repository without an option to turn them on again.
        if conf.get("archived") == [True]:
            return CheckResult.PASSED
        # GitHub enables the alerts on public repos but disables them on private repos by default.
        # is private repo
        if conf.get("private") == [True] or conf.get("visibility") in [["private"], ["internal"]]:
            if conf.get("vulnerability_alerts"):
                return CheckResult.PASSED
            return CheckResult.FAILED
        # is public repo
        if conf.get("vulnerability_alerts") == [False]:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = GithubRepositoryVulnerabilityAlerts()



from __future__ import annotations

import json
import logging
import os
from pathlib import Path
from typing import Any

import yaml
from jsonschema import validate, ValidationError

from checkov.common.parsers.yaml.loader import SafeLineLoaderGhaSchema
from checkov.common.parsers.yaml.parser import parse
from checkov.common.util.type_forcers import force_dict
from checkov.github_actions.schemas import gha_schema, gha_workflow

WORKFLOW_DIRECTORY = ".github/workflows/"


def get_scannable_file_paths(root_folder: str | Path) -> set[Path]:
    """Finds yaml files"""

    file_paths: set[Path] = set()

    if root_folder:
        root_path = root_folder if isinstance(root_folder, Path) else Path(root_folder)
        file_paths = {file_path for file_path in root_path.rglob("*.[y][am]*[l]") if file_path.is_file()}

    return file_paths


def parse_file(
    f: str | Path, file_content: str | None = None
) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
    file_path = f if isinstance(f, Path) else Path(f)

    if is_workflow_file(file_path):
        if not file_content:
            file_content = file_path.read_text()

        entity_schema = parse(filename=str(f), file_content=file_content)

        if entity_schema and is_schema_valid(yaml.load(file_content, Loader=SafeLineLoaderGhaSchema)):  # nosec
            return entity_schema
    return None


def is_workflow_file(file_path: str | Path) -> bool:
    """
    :return: True if the file mentioned is in a github action workflow directory and is a YAML file. Otherwise: False
    """
    abspath = os.path.abspath(file_path)
    return WORKFLOW_DIRECTORY in abspath and abspath.endswith(("yml", "yaml"))


def is_schema_valid(config: dict[str, Any] | list[dict[str, Any]]) -> bool:
    config_dict = force_dict(config)

    try:
        validate(config_dict, gha_workflow)
        return True
    except ValidationError:
        try:
            validate(config_dict, gha_schema)
            return True
        except ValidationError:
            logging.info(
                "Given entity configuration does not match the schema\n" f"config={json.dumps(config_dict, indent=4)}\n"
            )

    return False



from __future__ import annotations

from checkov.github.base_github_branch_security import BranchSecurity


class GithubBranchRequirePushRestrictions(BranchSecurity):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection requires push restrictions - CIS 1.1.15"
        id = "CKV_GITHUB_17"
        super().__init__(name=name, id=id)

    def get_evaluated_keys(self) -> list[str]:
        return ["restrictions"]


check = GithubBranchRequirePushRestrictions()



from checkov.common.models.enums import CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DynamodbGlobalTableRecovery(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Dynamodb global table point in time recovery (backup) is enabled"
        id = "CKV_AWS_165"
        supported_resources = ['AWS::DynamoDB::GlobalTable']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/Replicas/[0]/PointInTimeRecoverySpecification/PointInTimeRecoveryEnabled'


check = DynamodbGlobalTableRecovery()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


ADMIN_POLICY_NAME = "AdministratorAccess"
ADMIN_POLICY_ARN = f"arn:aws:iam::aws:policy/{ADMIN_POLICY_NAME}"


class IAMManagedAdminPolicy(BaseResourceCheck):
    def __init__(self) -> None:
        # This is the full description of your check
        description = "Disallow IAM roles, users, and groups from using the AWS AdministratorAccess policy"

        # This is the Unique ID for your check
        id = "CKV_AWS_274"

        # These are the terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_resources = (
            "aws_iam_role",
            "aws_iam_policy_attachment",
            "aws_iam_role_policy_attachment",
            "aws_iam_user_policy_attachment",
            "aws_iam_group_policy_attachment",
        )

        # Valid CheckCategories are defined in checkov/common/models/enums.py
        categories = (CheckCategories.IAM,)
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if self.entity_type == "aws_iam_role":
            if "managed_policy_arns" in conf.keys():
                if ADMIN_POLICY_ARN in conf["managed_policy_arns"][0]:
                    return CheckResult.FAILED

        elif self.entity_type in (
            "aws_iam_policy_attachment",
            "aws_iam_role_policy_attachment",
            "aws_iam_user_policy_attachment",
            "aws_iam_group_policy_attachment",
        ):
            policy_arn = conf.get("policy_arn")
            if policy_arn and policy_arn[0] == ADMIN_POLICY_ARN:
                return CheckResult.FAILED

        return CheckResult.PASSED


check = IAMManagedAdminPolicy()



from __future__ import annotations

from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.github.base_github_negative_branch_security import NegativeBranchSecurity


class GithubBranchDismissalRestrictions(NegativeBranchSecurity):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection restricts who can dismiss PR reviews - CIS 1.1.5"
        id = "CKV_GITHUB_12"
        super().__init__(name=name, id=id)

    def get_evaluated_keys(self) -> list[str]:
        return ["required_pull_request_reviews/dismissal_restrictions"]

    def get_forbidden_values(self) -> list[Any]:
        return [ANY_VALUE]


check = GithubBranchDismissalRestrictions()
